'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _typeof = typeof Symbol === "function" && typeof Symbol.iterator === "symbol" ? function (obj) { return typeof obj; } : function (obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; };

exports.default = function (cepRawValue) {
  return new _promiseAny2.default(function (resolve, reject) {
    _promiseAny2.default.resolve(cepRawValue).then(validateInputType).then(removeSpecialCharacters).then(validateInputLength).then(leftPadWithZeros).then(fetchCepFromServices).then(resolvePromise).catch(handleServicesError).catch(rejectWithCepPromiseError);

    function validateInputType(cepRawValue) {
      var cepTypeOf = typeof cepRawValue === 'undefined' ? 'undefined' : _typeof(cepRawValue);

      if (cepTypeOf === 'number' || cepTypeOf === 'string') {
        return cepRawValue;
      }

      throw new _cepPromise2.default({
        message: 'Erro ao inicializar a instância do CepPromise.',
        type: 'validation_error',
        errors: [{
          message: 'Você deve chamar o construtor utilizando uma String ou Number.',
          service: 'cep_validation'
        }]
      });
    }

    function removeSpecialCharacters(cepRawValue) {
      return cepRawValue.toString().replace(/\D+/g, '');
    }

    function leftPadWithZeros(cepCleanValue) {
      return '0'.repeat(CEP_SIZE - cepCleanValue.length) + cepCleanValue;
    }

    function validateInputLength(cepWithLeftPad) {
      if (cepWithLeftPad.length <= CEP_SIZE) {
        return cepWithLeftPad;
      }

      throw new _cepPromise2.default({
        message: 'CEP deve conter exatamente 8 caracteres.',
        type: 'validation_error',
        errors: [{
          message: 'CEP informado possui mais do que 8 caracteres.',
          service: 'cep_validation'
        }]
      });
    }

    function fetchCepFromServices(cepWithLeftPad) {
      return _promiseAny2.default.any([(0, _correios2.default)(cepWithLeftPad), (0, _viacep2.default)(cepWithLeftPad)]);
    }

    function resolvePromise(addressObject) {
      resolve(addressObject);
    }

    function handleServicesError(aggregatedErrors) {
      if (aggregatedErrors.length !== undefined) {
        throw new _cepPromise2.default({
          message: 'Todos os serviços de CEP retornaram erro.',
          type: 'service_error',
          errors: aggregatedErrors
        });
      }
      throw aggregatedErrors;
    }

    function rejectWithCepPromiseError(error) {
      reject(new _cepPromise2.default({
        message: error.message,
        type: error.type,
        errors: error.errors
      }));
    }
  });
};

var _correios = require('./services/correios.js');

var _correios2 = _interopRequireDefault(_correios);

var _viacep = require('./services/viacep.js');

var _viacep2 = _interopRequireDefault(_viacep);

var _cepPromise = require('./errors/cep-promise.js');

var _cepPromise2 = _interopRequireDefault(_cepPromise);

var _promiseAny = require('./utils/promise-any.js');

var _promiseAny2 = _interopRequireDefault(_promiseAny);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var CEP_SIZE = 8;

module.exports = exports['default'];