'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _isomorphicFetch = require('isomorphic-fetch');

var _isomorphicFetch2 = _interopRequireDefault(_isomorphicFetch);

var _service = require('../errors/service.js');

var _service2 = _interopRequireDefault(_service);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function fetchViaCepService(cepWithLeftPad) {

  return new Promise(function (resolve, reject) {
    var url = 'https://viacep.com.br/ws/' + cepWithLeftPad + '/json/';
    var options = {
      method: 'GET',
      mode: 'cors',
      headers: {
        'content-type': 'application/json;charset=utf-8'
      }
    };

    return (0, _isomorphicFetch2.default)(url, options).then(analyzeAndParseResponse).then(checkForViaCepError).then(extractCepValuesFromResponse).then(resolvePromise).catch(rejectWithServiceError);

    function analyzeAndParseResponse(response) {
      if (response.ok) {
        return response.json();
      }

      throw Error('Erro ao se conectar com o serviço ViaCEP.');
    }

    function checkForViaCepError(responseObject) {
      if (responseObject.erro === true) {
        throw new Error('CEP não encontrado na base do ViaCEP.');
      }

      return responseObject;
    }

    function extractCepValuesFromResponse(responseObject) {
      return {
        cep: responseObject.cep.replace('-', ''),
        state: responseObject.uf,
        city: responseObject.localidade,
        neighborhood: responseObject.bairro,
        street: responseObject.logradouro
      };
    }

    function resolvePromise(cepObject) {
      resolve(cepObject);
    }

    function rejectWithServiceError(error) {
      var serviceError = new _service2.default({
        message: error.message,
        service: 'viacep'
      });

      if (error.name === 'FetchError') {
        serviceError.message = 'Erro ao se conectar com o serviço ViaCEP.';
      }

      reject(serviceError);
    }
  });
}

exports.default = fetchViaCepService;
module.exports = exports['default'];